
#include "forms.h"
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <ib.h>
#include "drivers.h"
#include <string.h>

#define USE_SRQ 1
#define READS 5
#define WAIT 1
#define MAX_FREQ_PTS 51
#define MAX_DRIVE_PTS 30
#define MAX_RES 1024

extern int ibsta,ibcnt,iberr;
char cmd[130];
char res[MAX_RES];

/*
 * function to initialize and query model and parameters for RF sweeper
 */
SWEEPER_PAR *sweeper_init(int sweeper, int model)
{
  SWEEPER_PAR *parameters = (SWEEPER_PAR *) malloc(sizeof(*parameters));
  char flt_string[32];
  switch (model) {
    case 1:  /* Wiltron Sweeper */
      strcpy(cmd,"RST");
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      sleep(1);
      strcpy(cmd,"OI");
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      if( ibrd(sweeper,res,MAX_RES-1) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
  
      strncpy(flt_string,res+4,5);
      parameters->start_sweep = atof(flt_string);
  
      strncpy(flt_string,res+9,5);
      parameters->stop_sweep = atof(flt_string);
  
      strncpy(flt_string,res+14,6);
      parameters->rf_drive_lo = atof(flt_string);
      strncpy(flt_string,res+20,4);
      parameters->rf_drive_hi = atof(flt_string);
      
      if (parameters->rf_drive_hi - parameters->rf_drive_lo >= 35.0) {  
	strcpy(cmd,"IL1");   /* Use internal leveling with high dynamic range*/
	parameters->mtr_level = 0;
	if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
          gpiberr("wrt err");
          exit(1);
	}
      }
      else {   /* Use power meter leveling otherwise */
        strcpy(cmd,"PL1");
	parameters->mtr_level = 1;}
      
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      return parameters;
      
    case 2:  /* HP 8350 Sweeper */
      strcpy(cmd,"IP");  /* Instrument Preset (sweeps from low to high) */
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      sleep(1);
      
      strcpy(cmd,"RF0");   /* Turn RF off  */
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      strcpy(cmd,"OPFA");   /* Read start frequency (lowest) */
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      if( ibrd(sweeper,res,MAX_RES-1) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
      parameters->start_sweep = atof(res)/1e9;
  
      strcpy(cmd,"OPFB");   /* Read stop frequency (highest) */
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      if( ibrd(sweeper,res,MAX_RES-1) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
      parameters->stop_sweep = atof(res)/1e9;
  
      strcpy(cmd,"PL-99.0DM");   /* find lowest drive (trick) */
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      strcpy(cmd,"OPPL");
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      if( ibrd(sweeper,res,MAX_RES-1) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
      if (strncasecmp("--", res, 2)==0) {  /* Jury rig for Integra Output */
        parameters->rf_drive_lo = atof(res+1);}
      else {
        parameters->rf_drive_lo = atof(res);}
      
      strcpy(cmd,"PL+99.0DM");   /* find highest drive (trick) */
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      strcpy(cmd,"OPPL");
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      if( ibrd(sweeper,res,MAX_RES-1) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
      parameters->rf_drive_hi = atof(res);
      
      if (parameters->rf_drive_hi - parameters->rf_drive_lo >= 35.0) {  
	strcpy(cmd,"A1");   /* Use internal leveling with high dynamic range*/
	parameters->mtr_level = 0;
      }
      else {
        strcpy(cmd,"A3");  /* Use power meter leveling otherwise */
        parameters->mtr_level = 1;
      }
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      
      return parameters;
    default:
      return parameters;
    }
}

void set_sweeper_on(int sweeper,float freq, float drive, int model)
{
  switch (model) {
    case 1:  /* Wiltron Sweeper */
      sprintf(cmd,"CF0 F0%.3fGH LVL%2.fDM RF1\n", freq, drive);
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      return;
    case 2:  /* HP 8350 Sweeper */
      sprintf(cmd,"CW%.3fGZ PL%2.fDM RF1\n", freq, drive);
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      return;
    default:
  }
}

void set_rf_power(int sweeper, float drive, int model)
{
  switch (model) {
    case 1:  /* Wiltron Sweeper */
      sprintf(cmd,"LVL%2.fDM\n", drive);
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      return;
    case 2:  /* HP 8350 Sweeper */
      sprintf(cmd,"PL%2.fDM\n", drive);
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      return;
    default:
  }
}

void sweeper_rf_off(int sweeper, int model)
{
  switch (model) {
    case 1:  /* Wiltron Sweeper */
      sprintf(cmd,"RF0\n");
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      return;
    case 2:  /* HP 8350 Sweeper */
      sprintf(cmd,"RF0\n");
      if( ibwrt(sweeper,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      return;
    default:
  }
}

/*
 *   command meter to zero
 */
void mtr_zero(int meter,int model)
{
  char mtr_status[30];
  int i;
  switch (model) {
    case 1:  /* HP 437 */
      strcpy(cmd,"PRLGZE\n");
      if( ibwrt(meter,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
    case 2:  /* HP 436a */
      strcpy(cmd,"Z1T\n");
      if( ibwrt(meter,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      if( ibrd(meter,mtr_status,29) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
    default:
  }
}

/*
 *   check status of meter to see if zeroing operation is complete
 */
int mtr_zeroed(int meter,int model)
{
  char mtr_status[30], test[4];
  switch (model) {
    case 1:  /* HP 437 */
      strcpy(cmd,"SM\n");
      if( ibwrt(meter,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      if( ibrd(meter,mtr_status,29) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
      if (strncmp(mtr_status+4,"00",2) == 0) {return(1);}
      return(0);
    case 2:  /* HP 436a */
      strcpy(cmd,"Z1T\n");
      if( ibwrt(meter,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      if( ibrd(meter,mtr_status,29) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
      strncpy(test, mtr_status+4, 4);
      if (atoi(test) <= 2) {
	strcpy(cmd,"9D+T\n");
	if( ibwrt(meter,cmd,strlen(cmd)) & ERR ){
          gpiberr("wrt err");
          exit(1);
	}
	return(1);}
      else {return(0);}
    default:
  }
}

/*
 *   set meter to range and read power, return -1000 if error
 */
float pwr_set_range(int meter,int range,int model)
{
  char flt_string[32];
  switch (model) {
    case 1:  /* HP 437 */
      sprintf(cmd,"RM%dENTR2\n",range);
      if( ibwrt(meter,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      if( ibrd(meter,res,MAX_RES-1) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
      if (strncmp(res+9,"40",2) != 0) {
        strncpy(flt_string,res,11);
        return(atof(flt_string));
      }
      else {return(-1000.0);}
    case 2:  /* HP 436a */
      if (range == 0) {sprintf(cmd,"9D+T\n");}
      else {sprintf(cmd,"%dD+T\n",range);}
      if( ibwrt(meter,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      if( ibrd(meter,res,MAX_RES-1) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
      if (strncmp(res,"P",1) == 0) {
        return(atof(res+3));
      }
      else {return(-1000.0);}
    default:
  }
}

void set_range_rd(int meter,int range,int model)
{
  switch (model) {
    case 1:  /* HP 437 */
      if (range == 0) {sprintf(cmd,"RAENTR2\n");}
      else {sprintf(cmd,"RM%dENTR2\n",range);}
      if( ibwrt(meter,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
    case 2:  /* HP 436a */
      if (range == 0) {sprintf(cmd,"9D+T\n");}
      else {sprintf(cmd,"%dD+T\n",range);}
      if( ibwrt(meter,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
    default:
  }
}

float read_power(int meter, int model)
{
  char flt_string[32];
  int range;
  switch (model) {
    case 1:  /* HP 437 */
      if( ibrd(meter,res,MAX_RES-1) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
      if (strncmp(res+9,"40",2) != 0) {
        strncpy(flt_string,res,11);
        return(atof(flt_string));
      }
      else {
/*
 *   error, check for up range and try again
 */
        range = 0;
        while (range < 5) {
          strcpy(cmd,"SM\n");
          if( ibwrt(meter,cmd,strlen(cmd)) & ERR ){
            gpiberr("wrt err");
            exit(1);
          }
          if( ibrd(meter,res,MAX_RES-1) & ERR ){
            gpiberr("rd err");
            exit(1);
          }
          if (strncmp(res,"27",2) == 0) {return(-1000.0);}
          if (strncmp(res,"17",2) == 0) {
            range = read_range(meter, 1);
            range++;
            if (range > 5) {return(-1000.0);}
            set_range_rd(meter, range, 1);
            if( ibrd(meter,res,MAX_RES-1) & ERR ){
              gpiberr("rd err");
              exit(1);
            }
            if (strncmp(res+9,"40",2) != 0) {
              strncpy(flt_string,res,11);
              return(atof(flt_string));
            }
          }
          else {
            printf("%s\n",res); exit(1); }
        }
      }
    case 2:  /* HP 436a */
      if( ibrd(meter,res,MAX_RES-1) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
      if (strncmp(res,"P",1) == 0) {
        return(atof(res+3));
      }
      else {
/*
 *   error, check for up range and try again
 */
        range = 0;
        while (range < 5) {
          if (strncmp(res,"S",1) == 0) {return(-1000.0);}
          if (strncmp(res,"R",1) == 0) {
            range = read_range(meter, model);
            range++;
            if (range > 5) {return(-1000.0);}
            set_range_rd(meter, range, model);
            if( ibrd(meter,res,MAX_RES-1) & ERR ){
              gpiberr("rd err");
              exit(1);
            }
	    if (strncmp(res,"P",1) == 0) {
              return(atof(res+3));
	    }
          }
          else {
            printf("%s\n",res); exit(1); }
        }
      }
    default:
  }
}
       
int read_range(int meter, int model)
{
  char int_string[8];
  switch (model) {
    case 1:  /* HP 437 */
      strcpy(cmd,"SM\n");
      if( ibwrt(meter,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      if( ibrd(meter,res,MAX_RES-1) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
      strcpy(int_string,"   ");
      strncpy(int_string,res+7,1);
      return(atoi(int_string));
    case 2:  /* HP 436a */
      strcpy(cmd,"T\n");
      if( ibwrt(meter,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      if( ibrd(meter,res,MAX_RES-1) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
      return((int) *(res+1) - 72);
    default:
  }
}

void switch_path_a(int switch_driver, int model)
{
  switch (model) {
    case 1:
      strcpy(cmd,"a9\n");
      if( ibwrt(switch_driver,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
    default:
  }
}

void switch_path_b(int switch_driver, int model)
{
  switch (model) {
    case 1:
      strcpy(cmd,"b9\n");
      if( ibwrt(switch_driver,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
    default:
  }
}
