/*
 *    e-factory
 *
 *    Core of program designed to control all processes in a manufacturing
 *    environment.
 *    Components:
 *       Forms Library - ftp://laue.phys.uwm.edu/pub/xforms
 *       PLplot        - ftp://dino.ph.utexas.edu/plplot/
 *       GPIB          - ftp://ftp.llp.fu-berlin.de/pub/linux/LINUX-LAB/IEEE488/
 * 
 *    Programmer:	Danny Holstein
 *    Started:	1/13/97 (abandoned Visual BASIC/Windoze environment)
 */

#include "forms.h"
#include <stdlib.h>
#include "e_factory.h"
#include "e_fact.xpm"

FD_main *fd_main;
FD_LSG *fd_LSG;
FD_phase_velocity *fd_phase_velocity;
FD_DC_power *fd_DC_power;
FD_calibrate *fd_calibrate;
int CAL_check_exit();
int LSG_check_exit();

int main(int argc, char *argv[])
{
   fl_initialize(&argc, argv, 0, 0, 0);
   fd_main = create_form_main();

   /* show the first form */
   fl_show_form(fd_main->main,FL_PLACE_CENTER,FL_FULLBORDER,"e-factory");
   do {fl_do_forms();}
   while (fl_get_menu(fd_main->File_menu) != 4) ;
   return 0;
}

FD_main *create_form_main(void)
{
  FL_OBJECT *obj;
  FD_main *fdui = (FD_main *) fl_calloc(1, sizeof(*fdui));

  fdui->main = fl_bgn_form(FL_NO_BOX, 520, 230);
  obj = fl_add_box(FL_UP_BOX,0,0,520,230,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,250,50,250,130,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,20,50,200,130,"");
  obj = fl_add_box(FL_UP_BOX,0,0,520,40,"");
  fdui->File_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,10,10,50,20,"File");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_gravity(obj, FL_NorthWest, FL_NorthWest);
    fl_set_menu(obj, "Open|Save|Save As|Exit");
    fl_set_object_callback(obj,File_menu_cb,0);
  fdui->Edit_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,60,10,50,20,"Edit");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_gravity(obj, FL_NorthWest, FL_NorthWest);
    fl_set_object_callback(obj,Edit_menu_cb,0);
  fdui->Test_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,160,10,50,20,"Test");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_gravity(obj, FL_NorthWest, FL_NorthWest);
    fl_set_menu(obj, "Phase Velocity|Power");
    fl_set_object_callback(obj,Test_menu_cb,0);
  fdui->Help_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,460,10,50,20,"Help");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_gravity(obj, FL_NorthWest, FL_NorthWest);
    fl_set_menu(obj, "About|Contents");
    fl_set_object_callback(obj,Help_menu_cb,0);
  fdui->Equip_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,210,10,70,20,"Equipment");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_gravity(obj, FL_NorthWest, FL_NorthWest);
    fl_set_menu(obj, "RF Source|Vector Network Analyzer|Motor");
    fl_set_object_callback(obj,Equip_menu_cb,0);
  fdui->Calibrate_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,280,10,60,20,"Calibrate");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_gravity(obj, FL_NorthWest, FL_NorthWest);
    fl_set_object_callback(obj,Calibrate_menu_cb,0);
  fdui->Part = obj = fl_add_input(FL_NORMAL_INPUT,90,60,110,30,"Part");
  fdui->Engineer = obj = fl_add_input(FL_NORMAL_INPUT,90,100,110,30,"Engineer");
    fl_set_object_gravity(obj, FL_West, FL_NoGravity);
  fdui->Operator = obj = fl_add_input(FL_NORMAL_INPUT,90,140,110,30,"Operator");
    fl_set_object_gravity(obj, FL_West, FL_NoGravity);
  fdui->Process_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,110,10,50,20,"Process");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_gravity(obj, FL_NorthWest, FL_NorthWest);
    fl_set_object_callback(obj,Process_menu_cb,0);
  obj = fl_add_clock(FL_ANALOG_CLOCK,400,60,80,70,"");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
  obj = fl_add_pixmap(FL_NORMAL_PIXMAP,420,190,90,40,"");
    fl_set_pixmap_data(obj, e_fact_xpm);
  fdui->NC_file = obj = fl_add_input(FL_NORMAL_INPUT,330,140,150,30,"NC Test File");
    fl_set_object_callback(obj,NC_file_cb,0);
  fdui->NC_start = obj = fl_add_button(FL_NORMAL_BUTTON,260,60,110,30,"NC Test Start");
    fl_set_object_color(obj,FL_SPRINGGREEN,FL_PALEGREEN);
    fl_set_object_lcol(obj,FL_DARKTOMATO);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_callback(obj,NC_start_cb,0);
  fdui->NC_abort = obj = fl_add_button(FL_NORMAL_BUTTON,260,100,110,30,"Abort");
    fl_set_object_color(obj,FL_RED,FL_DARKTOMATO);
    fl_set_object_lcol(obj,FL_YELLOW);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_callback(obj,NC_abort_cb,0);
  fl_end_form();

  return fdui;
}

/*---------------------------------------*/


/* callbacks for form main */
void File_menu_cb(FL_OBJECT *ob, long data)
{
   switch (fl_get_menu(fd_main->File_menu)) {
      case 4:
         exit(0);
      default:
   }
}

void Edit_menu_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void Test_menu_cb(FL_OBJECT *ob, long data)
{
   switch (fl_get_menu(fd_main->Test_menu)) {
      case 1:
         fd_phase_velocity = create_form_phase_velocity();
         fl_hide_form(fd_main->main);
         fl_show_form(fd_phase_velocity->phase_velocity,FL_PLACE_CENTER,FL_FULLBORDER,"Phase Velocity");
         do {fl_do_forms();}
         while (fl_get_menu(fd_phase_velocity->VEL_file_menu) != 1) ;
         fl_hide_form(fd_phase_velocity->phase_velocity);
         fl_show_form(fd_main->main,FL_PLACE_CENTER,FL_FULLBORDER,"e-factory");
         fl_free_form(fd_phase_velocity->phase_velocity);
         break;
      case 2:
         fd_LSG = create_form_LSG();
         fl_hide_form(fd_main->main);
         fl_show_form(fd_LSG->LSG,FL_PLACE_CENTER,FL_FULLBORDER,"power");
         do {fl_do_forms();}
         while (LSG_check_exit() != 1);
         fl_hide_form(fd_LSG->LSG);
         fl_show_form(fd_main->main,FL_PLACE_CENTER,FL_FULLBORDER,"e-factory");
         fl_free_form(fd_LSG->LSG);
         if (fl_get_button(fd_LSG->DC_power)) {
           fl_hide_form(fd_DC_power->DC_power);
           fl_free_form(fd_DC_power->DC_power);
         }
         break;
      default:
         break;
   }
}


void DC_power_cb(FL_OBJECT *ob, long data)
{
  if (fl_get_button(ob)) {
    fd_DC_power = create_form_DC_power();
    fl_show_form(fd_DC_power->DC_power,FL_PLACE_CENTER,FL_FULLBORDER,"power supply control");
  }
  else {
    fl_hide_form(fd_DC_power->DC_power);
    fl_free_form(fd_DC_power->DC_power);
  }
}
void Help_menu_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void Equip_menu_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void Calibrate_menu_cb(FL_OBJECT *ob, long data)
{
  fd_calibrate = create_form_calibrate();
  fl_hide_form(fd_main->main);
  fl_show_form(fd_calibrate->calibrate,FL_PLACE_CENTER,FL_FULLBORDER,"calibrate couplers");
  do {fl_do_forms();}
  while (CAL_check_exit() != 1);
  fl_hide_form(fd_calibrate->calibrate);
  fl_show_form(fd_main->main,FL_PLACE_CENTER,FL_FULLBORDER,"e-factory");
  fl_free_form(fd_calibrate->calibrate);
}

void Process_menu_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void NC_file_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void NC_start_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void NC_abort_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

