
#include "forms.h"
#include <stdlib.h>
#include "plplot.h"
#include "e_factory.h"

FD_plot *fd_plot;
FL_OBJECT *obj;

void 
e_plot(int points, float x_min, float x_max, float y_min, float y_max,
     int x_ticks, int y_ticks, char *xlabel, char *ylabel, char *label,
     float xdata[], float ydata[], char *ps_file, int mode);

void 
fl_plot(int points, float x_min, float x_max, float y_min, float y_max,
     char *xlabel, char *ylabel, char *label,
     float xdata[], float ydata[])
{
    char qbuf[256];
    int x_ticks, y_ticks;
    
    fd_plot = create_form_plot();
/*
 *  load inputs with preliminary plot parameters
 */
    x_ticks = 10; y_ticks = 10;
    sprintf(qbuf, "%f", x_min);
    fl_set_input(fd_plot->PLOT_xmin, qbuf);
    sprintf(qbuf, "%f", x_max);
    fl_set_input(fd_plot->PLOT_xmax, qbuf);
    sprintf(qbuf, "%d", x_ticks);
    fl_set_input(fd_plot->PLOT_xticks, qbuf);
    sprintf(qbuf, "%f", y_min);
    fl_set_input(fd_plot->PLOT_ymin, qbuf);
    sprintf(qbuf, "%f", y_max);
    fl_set_input(fd_plot->PLOT_ymax, qbuf);
    sprintf(qbuf, "%d", y_ticks);
    fl_set_input(fd_plot->PLOT_yticks, qbuf);
    fl_set_input(fd_plot->PLOT_title, label);
    fl_set_input(fd_plot->PLOT_command, "lpr");
    
    fl_show_form(fd_plot->plot,FL_PLACE_CENTER,FL_FULLBORDER,"Display Data");
    while (obj = fl_do_forms()) {
    if (obj == fd_plot->PLOT_exit) {
       fl_hide_form(fd_plot->plot);
       fl_free_form(fd_plot->plot);
       break;}
    if (obj == fd_plot->PLOT_plot) {
       x_min = atof(fl_get_input(fd_plot->PLOT_xmin));
       x_max = atof(fl_get_input(fd_plot->PLOT_xmax));
       x_ticks = atof(fl_get_input(fd_plot->PLOT_xticks));
       y_min = atof(fl_get_input(fd_plot->PLOT_ymin));
       y_max = atof(fl_get_input(fd_plot->PLOT_ymax));
       y_ticks = atof(fl_get_input(fd_plot->PLOT_yticks));
       e_plot(points, x_min, x_max, y_min, y_max, x_ticks, y_ticks,
            xlabel, ylabel, fl_get_input(fd_plot->PLOT_title), xdata, ydata,
            "", 1);
       }
    if (obj == fd_plot->PLOT_printer) {
       x_min = atof(fl_get_input(fd_plot->PLOT_xmin));
       x_max = atof(fl_get_input(fd_plot->PLOT_xmax));
       y_min = atof(fl_get_input(fd_plot->PLOT_ymin));
       y_max = atof(fl_get_input(fd_plot->PLOT_ymax));
       e_plot(points, x_min, x_max, y_min, y_max, x_ticks, y_ticks,
            xlabel, ylabel, fl_get_input(fd_plot->PLOT_title), xdata, ydata,
            "file.ps", 2);
       }
    }
}

void 
e_plot(int points, float x_min, float x_max, float y_min, float y_max,
     int x_ticks, int y_ticks, char *xlabel, char *ylabel, char *label,
     float xdata[], float ydata[], char *ps_file, int mode)
{
    int space0 = 0, mark0 = 0, space1 = 1500, mark1 = 1500;
    char *command;
    command = malloc(80);
    
    switch (mode) {
      case 1:
        plsdev("xwin");
        break;
      case 2:
        plsdev("ps");
        plsfnam(ps_file);
        break;
      default:
        break;
     }

/* Initialize plplot */

    plinit();
    pladv(0);

/* Use standard viewport, and define X range from x_min to x_max, Y range
 * from y_min to y_max
 */
    plvsta();
    plwind(x_min, x_max, y_min, y_max);

/* Draw a box with ticks spaced 60 degrees apart in X, and 0.2 in Y. */

    plcol(1);
    plbox("bcnst", (x_max-x_min)/x_ticks, 2, "bcnstv", (y_max-y_min)/y_ticks, 2);

/* Superimpose a dashed line grid, with 1.5 mm marks and spaces. 
 * plstyl expects a pointer!! 
 */
    plstyl(1, &mark1, &space1);
    plcol(2);
    plbox("g", (x_max-x_min)/x_ticks, 0, "g", (y_max-y_min)/y_ticks, 0);
    plstyl(0, &mark0, &space0);

    plcol(3);
    pllab(xlabel, ylabel, label);

    plcol(4);
    plline(points, xdata, ydata);

/* Don't forget to call PLEND to finish off! */

    plend();
    if (mode == 2) {
      sprintf(command,"%s %s",fl_get_input(fd_plot->PLOT_command), ps_file);
      system(command);}
}

FD_plot *create_form_plot(void)
{
  FL_OBJECT *obj;
  FD_plot *fdui = (FD_plot *) fl_calloc(1, sizeof(*fdui));

  fdui->plot = fl_bgn_form(FL_NO_BOX, 420, 260);
  obj = fl_add_box(FL_UP_BOX,0,0,420,260,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,210,10,200,110,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,10,10,200,110,"");
  fdui->PLOT_xmin = obj = fl_add_input(FL_FLOAT_INPUT,110,20,90,30,"X Axis Minimum");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE+FL_EMBOSSED_STYLE);
  fdui->PLOT_xmax = obj = fl_add_input(FL_FLOAT_INPUT,110,50,90,30,"X Axis Maximum");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE+FL_EMBOSSED_STYLE);
  fdui->PLOT_ymin = obj = fl_add_input(FL_FLOAT_INPUT,310,20,90,30,"Y Axis Minimum");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE+FL_EMBOSSED_STYLE);
  fdui->PLOT_ymax = obj = fl_add_input(FL_FLOAT_INPUT,310,50,90,30,"Y Axis Maximum");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE+FL_EMBOSSED_STYLE);
  fdui->PLOT_title = obj = fl_add_input(FL_NORMAL_INPUT,40,140,360,30,"Title");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE+FL_EMBOSSED_STYLE);
  fdui->PLOT_plot = obj = fl_add_button(FL_NORMAL_BUTTON,20,180,110,30,"Plot");
    fl_set_object_color(obj,FL_SPRINGGREEN,FL_PALEGREEN);
    fl_set_object_lcol(obj,FL_DARKTOMATO);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE+FL_EMBOSSED_STYLE);
  fdui->PLOT_exit = obj = fl_add_button(FL_NORMAL_BUTTON,300,180,110,30,"Exit");
    fl_set_object_color(obj,FL_RED,FL_DARKTOMATO);
    fl_set_object_lcol(obj,FL_YELLOW);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE+FL_EMBOSSED_STYLE);
  fdui->PLOT_xticks = obj = fl_add_input(FL_INT_INPUT,110,80,90,30,"X Axis Ticks");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE+FL_EMBOSSED_STYLE);
  fdui->PLOT_yticks = obj = fl_add_input(FL_INT_INPUT,310,80,90,30,"Y Axis Ticks");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE+FL_EMBOSSED_STYLE);
  fdui->PLOT_printer = obj = fl_add_button(FL_NORMAL_BUTTON,160,180,110,30,"Plot to printer");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE+FL_EMBOSSED_STYLE);
  fdui->PLOT_command = obj = fl_add_input(FL_NORMAL_INPUT,160,220,250,30,"Print Command");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_NORMAL_STYLE+FL_EMBOSSED_STYLE);
  fl_end_form();

  return fdui;
}
