/*
 *    Program to probe helix TWT circuit to determine phase velocity
 *    and circuit loss.  Part of e-factory
 *    Programmer:	Danny Holstein
 *    Started:		1/21/97
 */ 

#include "forms.h"
#include <stdlib.h>
#include "e_factory.h"
#include <math.h>

#define MAX_AXIAL_PTS 200

FD_phase_velocity *fd_phase_velocity;

float freq[MAX_FREQ_PTS], velocity[MAX_FREQ_PTS];
float x[MAX_AXIAL_PTS], loss[MAX_AXIAL_PTS];
float pi;
int i, n, max_freq, max_pos;
void fl_plot(int points, float x_min, float x_max, float y_min, float y_max,
            char *xlabel, char *ylabel, char *label,
            float xdata[], float ydata[]);
void e_plot(int points, float x_min, float x_max, float y_min, float y_max,
            char *xlabel, char *ylabel, char *label,
            float xdata[], float ydata[]);

void VEL_start_cb(FL_OBJECT *ob, long data)
{
   float phase[MAX_FREQ_PTS][MAX_AXIAL_PTS];
   float magnitude[MAX_FREQ_PTS][MAX_AXIAL_PTS];
   float length, del_freq, del_x, f_start, f_stop;
   pi = asin(1.0)*2;
/*
 *    start button has been pressed, load input values
 */
  f_start = atof(fl_get_input(fd_phase_velocity->VEL_start_freq));
  f_stop = atof(fl_get_input(fd_phase_velocity->VEL_stop_freq));
  max_freq = atoi(fl_get_choice_text(fd_phase_velocity->VEL_freq_pts));
  length = atof(fl_get_input(fd_phase_velocity->VEL_movement));
  max_pos =  fl_get_counter_value(fd_phase_velocity->VEL_axial_pts);
/*
 *   initialize basic variables
 */
  del_freq = (f_stop - f_start)/(max_freq - 1);
  for(i=1; i<=max_freq; i++) {freq[i] = f_start + (i-1)*del_freq;}
  for(i=1; i<=max_freq; i++) {velocity[i] = .07;}
  del_x = length/(max_pos - 1);
  for(n=1; n<=max_pos; n++) {x[n] = (n-1)*del_x;}
  if (del_x==0. || del_freq<=0. || f_start==0. ) {
    for(i=1; i<=3; i++) {fl_set_menu_item_mode(fd_phase_velocity->VEL_display_menu, i, FL_PUP_GRAY);}
    return;}

/*
 *   initialize motor, vna, scale
 */
  for(i=1; i<=3; i++) {fl_set_menu_item_mode(fd_phase_velocity->VEL_display_menu, i, FL_PUP_NONE);}
}


void VEL_help_menu_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void VEL_display_menu_cb(FL_OBJECT *ob, long data)
{
   switch (fl_get_menu(fd_phase_velocity->VEL_display_menu)) {
      case 1:
         fl_plot(max_freq, freq[1], freq[max_freq], .05, .1, "Frequency (GHz)",
             "Velocity/C", "Tube Type", freq, velocity);
         break;
      case 2:
         fl_plot(max_pos, x[1], x[max_pos], -100., 0., "Distance", "Loss",
          "Tube Type",  x, loss);
         break;
      case 3:
         fl_plot(max_pos, x[1], x[max_pos], -100., 0., "Distance",
          "Differential Loss", "Tube Type",  x, loss);
         break;
      default:
         break;
   }
}

void VEL_abort_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

/*---------------------------------------*/

FD_phase_velocity *create_form_phase_velocity(void)
{
  int i;
  FL_OBJECT *obj;
  FD_phase_velocity *fdui = (FD_phase_velocity *) fl_calloc(1, sizeof(*fdui));

  fdui->phase_velocity = fl_bgn_form(FL_NO_BOX, 380, 300);
  obj = fl_add_box(FL_UP_BOX,0,0,380,300,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,190,140,180,150,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,10,50,180,80,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,10,140,180,150,"");
  obj = fl_add_box(FL_UP_BOX,0,0,380,40,"");
  fdui->VEL_file_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,10,10,60,20,"File");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_menu(obj, "Exit");
  fdui->VEL_display_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,90,10,60,20,"Display");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_menu(obj, "Phase Velocity|Loss|Differential Loss");
    for(i=1; i<=3; i++) {fl_set_menu_item_mode(obj, i, FL_PUP_GRAY);}
    fl_set_object_callback(obj,VEL_display_menu_cb,0);
  fdui->VEL_help_menu = obj = fl_add_menu(FL_PULLDOWN_MENU,310,10,60,20,"Help");
    fl_set_object_boxtype(obj,FL_FLAT_BOX);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_callback(obj,VEL_help_menu_cb,0);
  fdui->VEL_start_freq = obj = fl_add_input(FL_FLOAT_INPUT,100,60,80,30,"Start Frequency");
  fdui->VEL_stop_freq = obj = fl_add_input(FL_FLOAT_INPUT,100,90,80,30,"Stop Frequency");
  fdui->VEL_start = obj = fl_add_button(FL_NORMAL_BUTTON,280,50,70,30,"Start");
    fl_set_object_color(obj,FL_SPRINGGREEN,FL_PALEGREEN);
    fl_set_object_lcol(obj,FL_DARKTOMATO);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_callback(obj,VEL_start_cb,0);
  fdui->VEL_abort = obj = fl_add_button(FL_NORMAL_BUTTON,280,90,70,30,"Abort");
    fl_set_object_color(obj,FL_RED,FL_DARKTOMATO);
    fl_set_object_lcol(obj,FL_YELLOW);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_callback(obj,VEL_abort_cb,0);
  fdui->VEL_axial_pts = obj = fl_add_counter(FL_NORMAL_COUNTER,30,240,130,30,"Axial Points");
    fl_set_counter_bounds(obj,10,MAX_AXIAL_PTS);
    fl_set_counter_step(obj,1.0,10.0);
    fl_set_counter_precision(obj,0);
  fdui->VEL_cal = obj = fl_add_lightbutton(FL_PUSH_BUTTON,30,150,130,30,"User Calibration");
  fdui->VEL_carriage = obj = fl_add_button(FL_NORMAL_BUTTON,220,230,130,30,"Move Carriage");
  fdui->VEL_freq_pts = obj = fl_add_choice(FL_DROPLIST_CHOICE,30,190,130,30,"Frequencies");
    fl_set_object_boxtype(obj,FL_FRAME_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_BOTTOM);
    fl_addto_choice(obj,"51|101|201");
    fl_set_choice(obj, 2);
  fdui->VEL_movement = obj = fl_add_input(FL_FLOAT_INPUT,270,150,80,30,"Movement");
  fdui->VEL_offset = obj = fl_add_input(FL_FLOAT_INPUT,270,190,80,30,"Probe Offset");
  fl_end_form();

  return fdui;
}
