/*
 *    Program to drive amplifier power supply.  Part of e-factory
 *    Programmer:	Danny Holstein
 *    Started:		2/20/97
 */

#include "forms.h"
#include <stdlib.h>
#include "e_factory.h"
#include <math.h>
#include <stdio.h>
#include <unistd.h>
#include <ib.h>

#define USE_SRQ 1
#define READS 5
#define WAIT 1
#define MAX_RES 1024

extern int ibsta,ibcnt,iberr;
char cmd[130];
char res[MAX_RES];

FD_DC_power *fd_DC_power;

void set_dc_supply_on(float voltage, int model);
void set_dc_supply_off(int model);
float rd_dc_supply_cur(int model);
int dc_supply, dc_supply_model;
int twt_supply, twt_supply_model;

FD_DC_power *create_form_DC_power(void)
{
  FL_OBJECT *obj;
  FD_DC_power *fdui = (FD_DC_power *) fl_calloc(1, sizeof(*fdui));

  fdui->DC_power = fl_bgn_form(FL_NO_BOX, 580, 490);
  obj = fl_add_box(FL_UP_BOX,0,0,580,490,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,280,380,170,90,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,20,270,240,100,"");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
  obj = fl_add_frame(FL_ENGRAVED_FRAME,20,380,260,90,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,260,50,300,210,"");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,20,10,240,250,"");
  fdui->DC_voltage_w = obj = fl_add_input(FL_NORMAL_INPUT,60,70,70,30,"Helix");
  fdui->DC_voltage_on = obj = fl_add_input(FL_NORMAL_INPUT,60,130,70,30,"Grid");
  fdui->DC_voltage_off = obj = fl_add_input(FL_NORMAL_INPUT,60,160,70,30,"Bias");
  fdui->DC_voltage_a = obj = fl_add_input(FL_NORMAL_INPUT,60,190,70,30,"Anode");
  fdui->DC_button_on_g = obj = fl_add_checkbutton(FL_PUSH_BUTTON,130,130,30,30,"");
  fdui->DC_button_off_bias = obj = fl_add_checkbutton(FL_PUSH_BUTTON,130,160,30,30,"");
  fdui->DC_button_a = obj = fl_add_checkbutton(FL_PUSH_BUTTON,130,190,30,30,"");
  fdui->DC_cur_w = obj = fl_add_text(FL_NORMAL_TEXT,160,70,70,30,"0.00");
    fl_set_object_boxtype(obj,FL_DOWN_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  obj = fl_add_text(FL_NORMAL_TEXT,160,50,70,20,"Current");
    fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
  fdui->DC_voltage_c[1] = obj = fl_add_input(FL_NORMAL_INPUT,330,70,70,30,"Collector 1");
  fdui->DC_cur_col[1] = obj = fl_add_text(FL_NORMAL_TEXT,430,70,70,30,"0.00");
    fl_set_object_boxtype(obj,FL_DOWN_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  obj = fl_add_text(FL_NORMAL_TEXT,430,50,70,20,"Current");
    fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
  obj = fl_add_text(FL_NORMAL_TEXT,330,50,70,20,"Voltage");
    fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
  obj = fl_add_text(FL_NORMAL_TEXT,60,50,70,20,"Voltage");
    fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
  fdui->DC_voltage_c[2] = obj = fl_add_input(FL_NORMAL_INPUT,330,100,70,30,"Collector 2");
  fdui->DC_voltage_c[3] = obj = fl_add_input(FL_NORMAL_INPUT,330,130,70,30,"Collector 3");
  fdui->DC_voltage_c[4] = obj = fl_add_input(FL_NORMAL_INPUT,330,160,70,30,"Collector 4");
  fdui->DC_voltage_c[5] = obj = fl_add_input(FL_NORMAL_INPUT,330,190,70,30,"Collector 5");
  fdui->DC_voltage_f = obj = fl_add_input(FL_NORMAL_INPUT,60,100,70,30,"Heater");
  fdui->DC_pu = obj = fl_add_input(FL_NORMAL_INPUT,90,390,70,30,"Pulse Width");
  fdui->DC_rep = obj = fl_add_input(FL_NORMAL_INPUT,90,430,70,30,"Rep Rate");
  fdui->DC_cw = obj = fl_add_lightbutton(FL_PUSH_BUTTON,200,390,60,30,"CW");
    fl_set_object_callback(obj,DC_cw_cb,0);
  obj = fl_add_text(FL_NORMAL_TEXT,200,430,60,30,"0.00");
    fl_set_object_boxtype(obj,FL_DOWN_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  obj = fl_add_text(FL_NORMAL_TEXT,160,430,40,30,"Duty");
    fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
  fdui->DC_button_on_c[1] = obj = fl_add_checkbutton(FL_PUSH_BUTTON,400,70,30,30,"");
  fdui->DC_button_on_c[2] = obj = fl_add_checkbutton(FL_PUSH_BUTTON,400,100,30,30,"");
  fdui->DC_button_on_c[3] = obj = fl_add_checkbutton(FL_PUSH_BUTTON,400,130,30,30,"");
  fdui->DC_button_on_c[4] = obj = fl_add_checkbutton(FL_PUSH_BUTTON,400,160,30,30,"");
  fdui->DC_button_on_c[5] = obj = fl_add_checkbutton(FL_PUSH_BUTTON,400,190,30,30,"");
  fdui->DC_TWT = obj = fl_add_lightbutton(FL_PUSH_BUTTON,30,20,110,30,"Tube Supply");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
  fdui->DC_dc_ps = obj = fl_add_lightbutton(FL_PUSH_BUTTON,30,280,110,30,"DC Supply");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
  fdui->DC_vol_dc_ps = obj = fl_add_input(FL_FLOAT_INPUT,60,330,70,30,"Supply");
  fdui->DC_cur_dc_supply = obj = fl_add_text(FL_NORMAL_TEXT,160,330,70,30,"0.00");
    fl_set_object_boxtype(obj,FL_DOWN_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  obj = fl_add_text(FL_NORMAL_TEXT,160,310,70,20,"Current");
    fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
  obj = fl_add_text(FL_NORMAL_TEXT,60,310,70,20,"Voltage");
    fl_set_object_lalign(obj,FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
  fdui->DC_cur_htr = obj = fl_add_text(FL_NORMAL_TEXT,160,100,70,30,"0.00");
    fl_set_object_boxtype(obj,FL_DOWN_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->DC_cur_grid = obj = fl_add_text(FL_NORMAL_TEXT,160,130,70,30,"0.00");
    fl_set_object_boxtype(obj,FL_DOWN_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->DC_cur_bias = obj = fl_add_text(FL_NORMAL_TEXT,160,160,70,30,"0.00");
    fl_set_object_boxtype(obj,FL_DOWN_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->DC_cur_anode = obj = fl_add_text(FL_NORMAL_TEXT,160,190,70,30,"0.00");
    fl_set_object_boxtype(obj,FL_DOWN_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->DC_cur_col[2] = obj = fl_add_text(FL_NORMAL_TEXT,430,100,70,30,"0.00");
    fl_set_object_boxtype(obj,FL_DOWN_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->DC_cur_col[3] = obj = fl_add_text(FL_NORMAL_TEXT,430,130,70,30,"0.00");
    fl_set_object_boxtype(obj,FL_DOWN_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->DC_cur_col[4] = obj = fl_add_text(FL_NORMAL_TEXT,430,160,70,30,"0.00");
    fl_set_object_boxtype(obj,FL_DOWN_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->DC_cur_col[5] = obj = fl_add_text(FL_NORMAL_TEXT,430,190,70,30,"0.00");
    fl_set_object_boxtype(obj,FL_DOWN_BOX);
    fl_set_object_lalign(obj,FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
  fdui->DC_delay = obj = fl_add_input(FL_NORMAL_INPUT,350,390,70,30,"Test Delay (s)");
  fdui->DC_TWT_on = obj = fl_add_lightbutton(FL_PUSH_BUTTON,160,20,90,30,"Energize");
    fl_set_object_color(obj,FL_RED,FL_GREEN);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_callback(obj,DC_TWT_on_cb,0);
  fdui->DC_supply_on = obj = fl_add_lightbutton(FL_PUSH_BUTTON,160,280,90,30,"Energize");
    fl_set_object_color(obj,FL_RED,FL_GREEN);
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
    fl_set_object_lstyle(obj,FL_BOLD_STYLE);
    fl_set_object_callback(obj,DC_supply_on_cb,0);
  fdui->DC_htr_delay = obj = fl_add_input(FL_NORMAL_INPUT,330,20,70,30,"Htr Delay (s)");
  fdui->DC_htr_voltage_warm = obj = fl_add_input(FL_NORMAL_INPUT,470,20,70,30,"Warmup (V)");
  fdui->DC_cur_cathode = obj = fl_add_input(FL_NORMAL_INPUT,430,220,70,30,"Cathode Current");
  obj = fl_add_frame(FL_ENGRAVED_FRAME,260,10,300,40,"");
    fl_set_object_lsize(obj,FL_NORMAL_SIZE);
  fl_end_form();

  return fdui;
}
/*---------------------------------------*/
 
void DC_TWT_on_cb(FL_OBJECT *ob, long data)
{
  float voltage, current;
  float ew_voltage, iw_current;
  float ef_voltage, if_current;
  float eb_voltage[5], ib_current[5];
  float eg_voltage, ig_current;
  char value[15];
  twt_supply_model = atoi(parse("twt_supply","type"));
  if (fl_get_button(fd_DC_power->DC_TWT_on)) {
/*
 *   initialize twt supply
 */
    if((twt_supply = ibfind("twt_supply")) & ERR){
      gpiberr("ibfind err");
      exit(1);
    }
    if(ibclr(twt_supply) & ERR){
      gpiberr("clr err");
      exit(1);
    }
    if (fl_get_button(fd_DC_power->DC_TWT_on)) {
/*
 *   turn on supply if energize button is on, else, turn off
 */
      voltage = atof(fl_get_input(fd_DC_power->DC_voltage_f));
/*
 *       set_twt_supply_on(voltage, twt_supply_model);
 *       (void) rd_twt_supply_cur(twt_supply_model);
 */
    }
    else {
      /* set_twt_supply_off(1); */
      sprintf(value,"%f",0.0);
      fl_set_object_label(fd_DC_power->DC_cur_dc_supply, value);
    }
  }
}

void DC_supply_on_cb(FL_OBJECT *ob, long data)
{
  float voltage, current;
  char value[15];
  if (fl_get_button(fd_DC_power->DC_dc_ps)) {
/*
 *   initialize dc supply
 */
    if((dc_supply = ibfind("dc_supply")) & ERR){
      gpiberr("ibfind err");
      exit(1);
    }
    if(ibclr(dc_supply) & ERR){
      gpiberr("clr err");
      exit(1);
    }
    if (fl_get_button(fd_DC_power->DC_supply_on)) {
/*
 *   turn on supply if energize button is on, else, turn off
 */
      voltage = atof(fl_get_input(fd_DC_power->DC_vol_dc_ps));
      set_dc_supply_on(voltage, 1);
      (void) rd_dc_supply_cur(1);
    }
    else {
      set_dc_supply_off(1);
      sprintf(value,"%f",0.0);
      fl_set_object_label(fd_DC_power->DC_cur_dc_supply, value);
    }
  }
}
 
void DC_cw_cb(FL_OBJECT *ob, long data)
{
  /* fill-in code for callback */
}

void set_dc_supply_on(float voltage, int model)
{
  switch (model) {
    case 1:
      sprintf(cmd,"VSET %.3fV; ISET 20A; OUT ON;\n", voltage);
      if( ibwrt(dc_supply,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      sleep(1);
      return;
    default:
  }
}

float rd_dc_supply_cur(int model)
{
  float current;
  char value[15];
  switch (model) {
    case 1:
      sprintf(cmd,"IOUT?\n");
      if( ibwrt(dc_supply,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      if( ibrd(dc_supply,res,MAX_RES-1) & ERR ){
        gpiberr("rd err");
        exit(1);
      }
      sscanf(res,"IOUT %f",&current);
      sprintf(value,"%f", current);
      fl_set_object_label(fd_DC_power->DC_cur_dc_supply, value);
      return(current);
    default:
  }
}

void set_dc_supply_off(int model)
{
  switch (model) {
    case 1:
      sprintf(cmd,"OUT OFF;\n");
      if( ibwrt(dc_supply,cmd,strlen(cmd)) & ERR ){
        gpiberr("wrt err");
        exit(1);
      }
      return;
    default:
  }
}
