#include "forms.h"
#include <stdlib.h>
#include "e_factory.h"
#include <stdio.h>
#include <string.h>

char eq_value[80];

#include "util.h"

FD_LSG *fd_LSG;
char *tab = "\t";

char *parse(char *category, char *parameter)
{
  FILE *conf_file;
  char value[1024], message[1024];
  
  if ((conf_file = fopen(CONF_FILE,"r")) == NULL) {
    sprintf(message,"Couldn't open %s for reading\n", CONF_FILE);
    fl_show_message(message,"configuration required","");
    return NULL;}
  else {
    while (fscanf(conf_file ,"%s\n", value)
            != EOF ) {
      if (strncasecmp(value+1,category,strlen(category)) == 0) {
        while (fscanf(conf_file ,"%s\n", value)
            != EOF ) {
          if (strncasecmp(value,parameter,strlen(parameter)) == 0) {
            (void) strtok(value,"=");
            strcpy(eq_value,strtok(NULL,"="));
            break;
          }
        }
        break;
      }
    }
    (void) fclose(conf_file);
  }
  return eq_value;
}

/*
 *   read input and output calibrations from respective files
 */
int read_cal()
{
  int n_freq, channels, n;
  FILE *conf_file, *in_file, *out_file;
  char message[1024];
/*
 *   read input calibration
 */
  if ((in_file = fopen(fl_get_input(fd_LSG->LSG_in_cal),"r")) == NULL) {
    sprintf(message,"Couldn't open %s for reading\n", fl_get_input(fd_LSG->LSG_in_cal));
    fl_show_message(message,"Try new input calibration","");
    return 1;}
  else {
    n_freq=0;
    while (fscanf(in_file,"%f\t%f\n", &in_cal_freq[n_freq], &in_cal[n_freq])
            != EOF &&
           n_freq < MAX_FREQ_PTS) {n_freq++;}
    in_cal_pts = n_freq;
    (void) fclose(in_file);
  }
/*
 *   read output calibration
 */
  if ((out_file = fopen(fl_get_input(fd_LSG->LSG_out_cal),"r")) == NULL) {
    sprintf(message,"Couldn't open %s for reading\n", fl_get_input(fd_LSG->LSG_out_cal));
    fl_show_message(message,"Try new output calibration","");
    return 1;}
  else {
    channels = 0, n = 256;
    fgets(message, n, out_file);
    rewind(out_file);
    (void) strtok(message, tab);
    while (strtok(NULL, tab) != NULL) {channels++;}
    n_freq=0;
    if (channels == 1) {
      while (fscanf(out_file,"%f\t%f\n", &out_cal_freq[n_freq], &out_cal[n_freq][0])
              != EOF &&
             n_freq < MAX_FREQ_PTS) {n_freq++;}
      }
    if (channels == 2) {
      while (fscanf(out_file,"%f\t%f\t%f\n", &out_cal_freq[n_freq], &out_cal[n_freq][0],
                    &out_cal[n_freq][1])
              != EOF &&
             n_freq < MAX_FREQ_PTS) {n_freq++;}
      }
    out_cal_pts = n_freq;
    (void) fclose(out_file);
  }
  return (0);

}
/*
 *   read coupler factors
 */
int read_cplr_factors()
{
  int n_freq, n;
  FILE *conf_file, *in_file, *out_file;
  float InFactor, OutFactor;
  char message[1024];
  
  InFactor = atof(parse("couplers","InFactor"));
  OutFactor = atof(parse("couplers","OutFactor"));
  printf("Input %f \n", InFactor);
  printf("Output %f \n", OutFactor);
  if (InFactor == 0. || OutFactor == 0.) {
    sprintf(message,"non-zero factors required\n");
    fl_show_message(message,"check configuration","");
    return 1;}
/*
 *   read configuration file for coupler calibrations
 */
  if ((in_file = fopen(parse("couplers","InCalFile"),"r")) == NULL) {
    sprintf(message,"Couldn't open %s for reading\n", parse("couplers","InCalFile"));
    fl_show_message(message,"coupler calibration required","");
    return 1;}
  else {
    n_freq=0;
    while (fscanf(in_file,"%f\t%f\n", &in_cplr_freq[n_freq], &in_cplr[n_freq])
            != EOF &&
           n_freq < MAX_FREQ_PTS) {
      in_cplr[n_freq] = in_cplr[n_freq] * InFactor;
      n_freq++;}
    in_cplr_pts = n_freq;
    (void) fclose(in_file);
  }
  if ((out_file = fopen(parse("couplers","OutCalFile"),"r")) == NULL) {
    sprintf(message,"Couldn't open %s for reading\n", parse("couplers","OutCalFile"));
    fl_show_message(message,"coupler calibration required","");
    return 1;}
  else {
    n_freq=0;
    while (fscanf(out_file,"%f\t%f\n", &out_cplr_freq[n_freq], &out_cplr[n_freq])
            != EOF &&
           n_freq < MAX_FREQ_PTS) {
      out_cplr[n_freq] = out_cplr[n_freq] * OutFactor;
      n_freq++;}
    out_cplr_pts = n_freq;
    (void) fclose(in_file);
  }
  return (0);

}

/*
 *   read drive levels
 */
int read_drives()
{
  int n_freq;
  FILE *drive_file;
  char message[1024];
/*
 *   read output calibration
 */
  if ((drive_file = fopen(fl_get_input(fd_LSG->LSG_drive_box),"r")) == NULL) {
    sprintf(message,"Couldn't open %s for reading\n", fl_get_input(fd_LSG->LSG_drive_box));
    fl_show_message(message,"Try new file","");
    return 1;}
  else {
    n_freq=0;
    while (fscanf(drive_file,"%f\t%f\t%f\n", &in_drive_freqs[n_freq], &in_drive_start[n_freq],
                  &in_drive_stop[n_freq])
            != EOF &&
           n_freq < MAX_FREQ_PTS) {n_freq++;}
    drive_pts = n_freq;
    (void) fclose(drive_file);
  }
  return (0);

}
